/*
 * $Id: load.c,v 1.5 1998/12/21 07:07:45 masaki Exp $
 */

#include <mrt.h>

static bgp_peer_t *create_fake_peer ();

/* load_rib_from_disk
 * Mainly used in testing -- load a binary routing table dump
 * into our RIB
 */
void load_rib_from_disk (char *filename) {
  mrt_msg_t *msg;
  io_t *io;
  int i, len;
  u_char *cp, status, mask;
  u_short view_num, seq_num, attrlen;
  u_long prefix_long, originated;
  prefix_t *prefix;
  bgp_peer_t *peer;
  view_t *view;

  io = New_IO (MRT->trace);
  if (io_set (io, IO_INFILE, (char *)filename, 0) < 0) {
    return;
  }

  while (1) {
    if ((msg = (mrt_msg_t *) io_read (io)) == NULL) {
      /*       printf("\nInvalid MSG "); */
      return;
    }

    if (msg->type != MSG_TABLE_DUMP) 
      continue;

    len = msg->length;
    cp = msg->value;
    BGP_GET_SHORT (view_num, cp);
    BGP_GET_SHORT (seq_num, cp);
    len -= 2;

    while (len > 0) {
      BGP_GET_ADDR (&prefix_long, cp);
      BGP_GET_BYTE (mask, cp);
      BGP_GET_BYTE (status, cp);
      BGP_GET_LONG (originated, cp);
      BGP_GET_SHORT (attrlen, cp);

      prefix = New_Prefix (AF_INET, (u_char *) &prefix_long, mask);
      
      peer = create_fake_peer ();
      bgp_munge_attributes (attrlen, cp, peer);

      /* need to create gateway */
      peer->attr->gateway = ref_nexthop (peer->attr->nexthop);

      for (i = 0; i < MAX_BGP_VIEWS; i++) {
	if ((view = BGP->views[i]) == NULL ||
	    view->family != prefix->family)
	  continue;

	view_open (view);
	bgp_add_route (view, prefix, peer->attr);
      
	bgp_process_changes (view);
	view_close (view);
      }

      /* I still don't understand when we can delete prefixes */
      Deref_Prefix (prefix); 

      if (peer->attr) 
	bgp_deref_attr (peer->attr);
      Delete (peer);
      cp += attrlen;
      len -= attrlen;
      len -= 12;
    }
  }
}

static bgp_peer_t *create_fake_peer () {
  bgp_peer_t *peer;

  peer = New (bgp_peer_t);
  peer->aliases = NULL;
  peer->sockfd = -1;
  peer->sockfd_in = -1;
  peer->listen_socket = NULL;

  peer->dlist_in = peer->dlist_out = -1;
  peer->flist_in = peer->flist_out = -1;
  peer->route_map_in = peer->route_map_out = -1;

  peer->read_ptr = peer->buffer;
  peer->start_ptr = peer->buffer;
  peer->packet = NULL;
  peer->read_ptr_in = peer->buffer_in;

  peer->state = BGPSTATE_IDLE;

  peer->maximum_prefix = 0;

  peer->Start_Interval = -1;
  peer->ConnectRetry_Interval = -1;
  peer->KeepAlive_Interval = -1;
  peer->HoldTime_Interval = -1;

  return (peer);
}
